<?php
namespace Teknisa\CTB\Modules\Saldos\Services;

use Teknisa\Libs\Util\Environment;
use Teknisa\CTB\Modules\Saldos\Factories\RecalculoSaldosFactory;
use Zeedhi\Framework\ORM\DateTime;

class RecalculoSaldosService {
    
    const DATA_INICIO_FORA_PERIODO_CONTABIL = 'DATA_INICIO_FORA_PERIODO_CONTABIL';
    const CAMPO_VAZIO = 'CAMPO_VAZIO'; 
    const DATA_NAO_POSSUI_MES  = 'DATA_NAO_POSSUI_MES';
    
    protected $recalculoSaldosFactory;
    protected $nrorg;
    
    public function __construct(Environment $environment, RecalculoSaldosFactory $recalculoSaldosFactory) {
        $this->nrorg = $environment->getNrorgTrab();
        $this->recalculoSaldosFactory = $recalculoSaldosFactory;
    }
    
    private function getfistDayOfDate($date){
        $firstDay = DateTime::createFromFormat('m/Y', $date);
        if($firstDay === null){
            $firstDay = DateTime::createFromFormat('d/m/Y', $date);
            if($firstDay === null){
                throw new \Exception(self::DATA_NAO_POSSUI_MES);
            }
        }
        $firstDay->modify('first day of this month');
        return $firstDay->format('d/m/Y');
        
    }
    
    private function getMonthYearOfDate($date){
        $monthYearDate = DateTime::createFromFormat('m/Y', $date);
        if($monthYearDate === null){
            $monthYearDate = DateTime::createFromFormat('d/m/Y', $date);
            if($monthYearDate === null){
                throw new \Exception(self::DATA_NAO_POSSUI_MES);
            }
        }
        return $monthYearDate->format('m/Y');
    }
    
    public function recalcularSaldo($row){
        $datainic = self::getfistDayOfDate($row['DTSALDO']);
        $this->recalculoSaldosFactory->callSaldoGeralProcedure($row['CDEMPRESA'], $datainic);
    }
    
    public function validatePeriodoContabil($row){
        $datainic = self::getMonthYearOfDate($row['DTSALDO']);
		if(!$this->recalculoSaldosFactory->getEmpresaFromPeriodoContabil($row['CDEMPRESA'], $row['DTSALDO'], $this->nrorg))
		    throw new \Exception(self::DATA_FORA_PERIODO_CONTABIL);
	}
	
    public function validateRow($row){
        if(!isset($row['CDEMPRESA']) || !isset($row['DTSALDO'])){
            throw new \Exception(self::CAMPO_VAZIO);
        }
    }
	
}